﻿using Microsoft.Extensions.Logging;
using PizzeriaContracts.BindingModels;
using PizzeriaContracts.BusinessLogicsContracts;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StorageContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels.Enums; 

namespace PizzeriaBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;

        private readonly IOrderStorage _orderStorage;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }
        
        public bool CreateOrder(OrderBindingModel model)
        {
            
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }
            model.Status = OrderStatus.Принят;
            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderId: {Id}.", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выполняется);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }
        public bool ChangeStatus(OrderBindingModel model, OrderStatus status)
        {
            CheckModel(model, false);
            
            var searchOrder = new OrderSearchModel { Id = model.Id };
            var order = _orderStorage.GetElement(searchOrder);
            
            if (order == null)
            {
                throw new ArgumentNullException(nameof(order));
            }
            model.PizzaId = order.PizzaId;
            model.Count = order.Count;
            model.Sum = order.Count;
            model.DateCreate = order.DateCreate;
            model.DateImplement = order.DateImplement;
            model.Status = order.Status;

            if (model.Status != status - 1)
            {
                _logger.LogWarning("Status update to " + status + " operation failed");
                return false;
            }
            model.Status = status;

            if (model.Status == OrderStatus.Готов)
            {
                model.DateImplement = DateTime.Now;
            }
            if (_orderStorage.Update(model) == null)
            {
                model.Status--;
                _logger.LogWarning("Changing status operation faled");
                return false;
            }
            return true;
 
        }
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.PizzaId <= 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор пиццы", nameof(model.PizzaId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("В заказе должно быть хотя бы одно пицца", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Стоимость заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. Id: {Id}. Sum: {Sum}. PizzaId: {PizzaId}. ReinforcedCount: {Count}", model.Id, model.Sum, model.PizzaId, model.Count);
        }
    }
}
